/**
  ******************************************************************************
  * @file    hw_config.c
  * @author  MCD Application Team
  * @version V4.0.0
  * @date    21-January-2013
  * @brief   Hardware Configuration & Setup
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2013 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software 
  * distributed under the License is distributed on an "AS IS" BASIS, 
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */


/* Includes ------------------------------------------------------------------*/
#include "hw_config.h"
#include "usb_lib.h"
#include "usb_pwr.h"
#include "stm32f10x_gpio.h"
#include "misc.h"
#include "stm32f10x_pwr.h"
#include "stm32f10x_exti.h"
#include "stm32f10x_rcc.h"
#include "hardwareconfig.h"
#include "debughelp.h"



/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
ErrorStatus HSEStartUpStatus;
EXTI_InitTypeDef EXTI_InitStructure;

/* Extern variables ----------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/*******************************************************************************
* Function Name  : Set_System
* Description    : Configures Main system clocks & power.
* Input          : None.
* Return         : None.
*******************************************************************************/
// Eigene Modifikationen: Angepasst nur fuer STM32F103C8
// + Beschraenkung auf vorhandene Hardware
void Set_System(void)
{

	GPIO_InitTypeDef  GPIO_InitStructure;
  
	/*!< At this stage the microcontroller clock setting is already configured,
       this is done through SystemInit() function which is called from startup
       file (startup_stm32xxx.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_stm32xxx.c file
     */ 


  	{
		/* USB_DISCONNECT used as USB pull-up */
		#ifdef USB_PULLUP_GPIO
			GPIO_InitTypeDef GPIO_InitStructure;
			RCC_APB2PeriphClockCmd(USB_PULLUP_GPIO_CLK, ENABLE);

			GPIO_InitStructure.GPIO_Pin = USB_PULLUP_Pin;
			GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
			GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
			GPIO_Init(USB_PULLUP_GPIO, &GPIO_InitStructure);

			GPIO_SetBits(USB_PULLUP_GPIO, USB_PULLUP_Pin);
		#endif
  	}

  	{
  		/* Pins fuer Signal-LEDs */
  		GPIO_InitTypeDef GPIO_InitStructure;
  		GPIO_InitStructure.GPIO_Pin = ERROR_LED_Pin;
  		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  		GPIO_Init(ERROR_LED_GPIO, &GPIO_InitStructure);
  	}


    /* Configure the EXTI line 18 connected internally to the USB IP **********/
    EXTI_ClearITPendingBit(EXTI_Line18);
    EXTI_InitStructure.EXTI_Line = EXTI_Line18;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);

}

/*******************************************************************************
* Function Name  : Set_USBClock
* Description    : Configures USB Clock input (48MHz).
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
void Set_USBClock(void)
{
    /* Select USBCLK source */
	switch(SystemCoreClock) {
		case 48000000UL:
			RCC_USBCLKConfig(RCC_USBCLKSource_PLLCLK_Div1);
		break;
		case 72000000UL:
			RCC_USBCLKConfig(RCC_USBCLKSource_PLLCLK_1Div5);
			break;
		default:
			// wird nicht funktionieren
			error();
	}
  
  	/* Enable the USB clock */
  	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USB, ENABLE);
}


/*******************************************************************************
* Function Name  : Enter_LowPowerMode.
* Description    : Power-off system clocks and power while entering suspend mode.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
void Enter_LowPowerMode(void)
{
	/* Set the device state to suspend */
	bDeviceState = SUSPENDED;

	/* Request to enter STOP mode with regulator in low power mode */
	PWR_EnterSTOPMode(PWR_Regulator_LowPower, PWR_STOPEntry_WFI);
}


/*******************************************************************************
* Function Name  : Leave_LowPowerMode.
* Description    : Restores system clocks and power while exiting suspend mode.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
void Leave_LowPowerMode(void)
{
  DEVICE_INFO *pInfo = &Device_Info;
  
  /* Set the device state to the correct state */
  if (pInfo->Current_Configuration != 0)
  {
    /* Device configured */
    bDeviceState = CONFIGURED;
  }
  else
  {
    bDeviceState = ATTACHED;
  }
  
  /*Enable SystemCoreClock*/
  SystemInit();
}

/*******************************************************************************
* Function Name  : USB_Interrupts_Config.
* Description    : Configures the USB interrupts.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
void USB_Interrupts_Config(void)
{
    NVIC_InitTypeDef NVIC_InitStructure;

    /* 2 bit for pre-emption priority, 2 bits for subpriority */
    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);
 
    /* Enable the USB interrupt */
    NVIC_InitStructure.NVIC_IRQChannel = USB_LP_CAN1_RX0_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 2;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    /* Enable the USB Wake-up interrupt */
    NVIC_InitStructure.NVIC_IRQChannel = USBWakeUp_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
    NVIC_Init(&NVIC_InitStructure);
}


/*******************************************************************************
* Function Name  : USB_Cable_Config.
* Description    : Software Connection/Disconnection of USB Cable.
* Input          : NewState: new state.
* Output         : None.
* Return         : None
*******************************************************************************/
void USB_Cable_Config (FunctionalState NewState)
{
	// Das Eval-Board kann den Pull-Up-Zustand nicht beinflussen

	#ifdef USB_PULLUP_GPIO
		GPIO_InitTypeDef GPIO_InitStructure;
		GPIO_InitStructure.GPIO_Pin = USB_PULLUP_Pin;
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;

		if (NewState != DISABLE)
		{
			// Pull-Up aktiv
			GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
			GPIO_Init(USB_PULLUP_GPIO, &GPIO_InitStructure);
			GPIO_SetBits(USB_PULLUP_GPIO, USB_PULLUP_Pin);
			//GPIO_ResetBits(USB_PULLUP_GPIO, USB_PULLUP_Pin);
		}
		else
		{
			// Pull-Up inaktiv
			GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
			GPIO_Init(USB_PULLUP_GPIO, &GPIO_InitStructure);
			//GPIO_SetBits(USB_PULLUP_GPIO, USB_PULLUP_Pin);
		}
	#endif
}






/************************ (C) COPYRIGHT ??? *****END OF FILE****/
