/*
 * inputkeys.c
 *
 * Konfiguration von Eingangs-Tastern (Pins am C und die auszuloesenden Tasten-
 * kombinationen).
 *
 * Werden mehrere Tasten gedrueckt, gewinnt die Taste mit der hoechsten
 * Prioritaet (niedrigster Zahlenwert fuer "prio"). Haben mehrere Tasten
 * dieselbe hoechste Prioritaet, werden sie gleichzeitig gesendet (bis zu 6
 * Tasten)
 *
 * Liegen mehrere Struct-Elemente aus Inputkeyconf[] auf demselben Taster,
 * werden alle Tasten (mit der normalen Priorisierung) ausgeloest.
 *
 * Konfiguration in inputkey.c, in inputkeys.h sind keine Einstellungen
 * vorzunehmen. Bei Bedarf koennen allerdings Tasten ergaenzt werden.
 *
 * Es wird direkt die HID-Usage-ID der Taste hinterlegt. Das erspart die
 * Implementierung einer kompletten Keymap.
 *
 * 18.09.2014 Nicolas
 */


#include <stdint.h>
#include <stdbool.h>
#include "inputkeys.h"


/* Konfiguration aller Eingabetaster, ihrer Funktion und ihrer Prioritaet */
const inputkeyconf_t Inputkeyconf[] =
{
	{   // Stop
		.GPIOx    = GPIOA,
		.GPIO_Pin = GPIO_Pin_0,
		.prio     = 1,
		.modifier = LEFT_ALT,
		.key      = KEY_S,
		.active_modes = 0xFF,
	},

	{   // Pause, feed hold
		.GPIOx    = GPIOA,
		.GPIO_Pin = GPIO_Pin_1,
		.prio     = 1,
		.modifier = none,
		.key      = KEY_SPACE,
		.active_modes = 0xFF,
	},
	{   // Cycle start
		.GPIOx    = GPIOA,
		.GPIO_Pin = GPIO_Pin_2,
		.prio     = 50,
		.modifier = LEFT_ALT,
		.key      = KEY_R,
		.active_modes = 0xFF,
	},
	{  	// rewind
		.GPIOx    = GPIOA,
		.GPIO_Pin = GPIO_Pin_3,
		.prio     = 5,
		.modifier = LEFT_CTRL,
		.key      = KEY_W,
		.active_modes = 0xFF,
	},
	{  	// feed lower
		.GPIOx    = GPIOA,
		.GPIO_Pin = GPIO_Pin_4,
		.prio     = 2,
		.modifier = none,
		.key      = KEY_F10,
		.active_modes = 0xFF,
	},
	{  	// feed raise
		.GPIOx    = GPIOA,
		.GPIO_Pin = GPIO_Pin_5,
		.prio     = 11,
		.modifier = none,
		.key      = KEY_F11,
		.active_modes = 0xFF,
	},
	{  	// Spindle On/Off
		.GPIOx    = GPIOA,
		.GPIO_Pin = GPIO_Pin_6,
		.prio     = 2,
		.modifier = none,
		.key      = KEY_F5,
		.active_modes = 0xFF,
	},
	{  	// Flood On/Off
		.GPIOx    = GPIOA,
		.GPIO_Pin = GPIO_Pin_7,
		.prio     = 3,
		.modifier = LEFT_CTRL,
		.key      = KEY_F,
		.active_modes = 0xFF,
	},
	{  	// Jog On/Off
		.GPIOx    = GPIOA,
		.GPIO_Pin = GPIO_Pin_8,
		.prio     = 10,
		.modifier = LEFT_CTRL | LEFT_ALT,
		.key      = KEY_F,
		.active_modes = 0xFF,
	},
	{    // Achse X (eigene Kombination)
		.GPIOx    = GPIOA,
		.GPIO_Pin = GPIO_Pin_9, // PA9 ist "TX" beschriftet auf dem Eval-Board
		.prio     = 50,
		.modifier = LEFT_ALT,
		.key      = KEY_X,
		.active_modes = 0xFF,
	},
	{   // Achse Y (eigene Kombination)
		.GPIOx    = GPIOA,
		.GPIO_Pin = GPIO_Pin_10, // PA10 ist "RX" beschriftet auf dem Eval-Board
		.prio     = 51,
		.modifier = LEFT_ALT,
		.key      = KEY_Z, // bei einer deutschen Tastatur sind Y und Z vertauscht
		.active_modes = 0xFF,
	},
	{    // Achse Z (eigene Kombination)
		.GPIOx    = GPIOB,
		.GPIO_Pin = GPIO_Pin_8,
		.prio     = 52,
		.modifier = LEFT_ALT,
		.key      = KEY_Y, // bei einer deutschen Tastatur sind Y und Z vertauscht
		.active_modes = 0xFF,
	},
	{   // Achse A (eigene Kombination)
		.GPIOx    = GPIOB,
		.GPIO_Pin = GPIO_Pin_9,
		.prio     = 53,
		.modifier = LEFT_ALT,
		.key      = KEY_A,
		.active_modes = 0xFF,
	},
	{   // Pfeil links
		.GPIOx    = GPIOB,
		.GPIO_Pin = GPIO_Pin_10,
		.prio     = 50,
		.modifier = none,
		.key      = KEY_LEFT_ARROW,
		.active_modes = 0xFF,
	},
	{   // Pfeil rechts
		.GPIOx    = GPIOB,
		.GPIO_Pin = GPIO_Pin_11,
		.prio     = 50,
		.modifier = none,
		.key      = KEY_RIGHT_ARROW,
		.active_modes = 0xFF,
	},
	{   // Pfeil hoch
		.GPIOx    = GPIOB,
		.GPIO_Pin = GPIO_Pin_12,
		.prio     = 50,
		.modifier = none,
		.key      = KEY_UP_ARROW,
		.active_modes = 0xFF,
	},
	{   // Pfeil herunter
		.GPIOx    = GPIOB,
		.GPIO_Pin = GPIO_Pin_13,
		.prio     = 50,
		.modifier = none,
		.key      = KEY_DOWN_ARROW,
		.active_modes = 0xFF,
	},
	{   // Bild auf
		.GPIOx    = GPIOB,
		.GPIO_Pin = GPIO_Pin_14,
		.prio     = 50,
		.modifier = none,
		.key      = KEY_PAGE_UP,
		.active_modes = 0xFF,
	},
	{   // Bild ab
		.GPIOx    = GPIOB,
		.GPIO_Pin = GPIO_Pin_15,
		.prio     = 50,
		.modifier = none,
		.key      = KEY_PAGE_DOWN,
		.active_modes = 0xFF,
	},
	{   // Shift + Pfeil links
		.GPIOx    = GPIOC,
		.GPIO_Pin = GPIO_Pin_13,
		.prio     = 50,
		.modifier = LEFT_SHIFT,
		.key      = KEY_LEFT_ARROW,
		.active_modes = 0xFF,
	},
	/*
	 * PC14 + PC5 haengen am Eval-Board an einem Quarz, der erst entfernt werden
	 * muesste
	{   // Shift + Pfeil rechts
		.GPIOx    = GPIOC,
		.GPIO_Pin = GPIO_Pin_14,
		.prio     = 50,
		.modifier = LEFT_SHIFT,
		.key      = KEY_RIGHT_ARROW,
		.active_modes = 0xFF,
	},
	*/
	/* Solange BOOT0 low ist, kann PB2/BOOT als normaler I/O benutzt werden
	{
		.GPIOx    = GPIOB,
		.GPIO_Pin = GPIO_Pin_2,
		.prio     = 50,
		.modifier =  ,
		.key      =  ,
		.active_modes = 0xFF,
	},
	*/
};
// Belegt:
// PA11, PA12, PA13, PA14, PA15
// PB0 - PB7 (SPARE1)
// (PC15, PC14)


uint8_t keybuffer[8] = {0,0,0,0,0,0,0,0};
const uint32_t nInputkey = sizeof(Inputkeyconf)/sizeof(Inputkeyconf[0]);
#define N_INPUTKEY (sizeof(Inputkeyconf)/sizeof(Inputkeyconf[0]))


/* Alle Tasten inititalisieren
 * Es wird davon ausgegangen, das der GPIO-Clock schon laeuft */
void inputkeys_init(void)
{
	GPIO_TypeDef* GPIOx;
	GPIO_InitTypeDef GPIO_InitStructure;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;

	for(uint32_t i = 0;i<nInputkey;i++) {
		GPIO_InitStructure.GPIO_Pin = Inputkeyconf[i].GPIO_Pin;
		GPIOx                       = Inputkeyconf[i].GPIOx;
		GPIO_Init(GPIOx, &GPIO_InitStructure);
	}
}


/* Keyboard-Puffer leeren */
static inline void clearKeybuffer(void)
{
	memset(keybuffer,0,8);
}


/* Tasten scannen
 * return: Gueltiger HID-Keyboard-buffer
 *
 * Funktion ist nicht besonders optimiert, fuer den Anwendungszweck jedoch
 * absolut ausreichend */
uint8_t *inputkeys_scan(void)
{
	static uint_fast8_t debouncebuf[N_INPUTKEY];
	uint_fast8_t ikey = 0;      // Index aktuell gedrueckter Taste
	uint_fast8_t hiPrio = 0xFF; // bislang hoechste aktive Prioritaet

	clearKeybuffer();

	for(uint32_t i = 0;i<nInputkey;i++) {
		bool isKeypress; // Tastendruck erkannt
		GPIO_TypeDef* GPIOx;
		uint16_t GPIO_Pin;

		// Tastenstatus lesen
		// Einfache Entprellung (active low, vierfache Abtastung)
		GPIOx    = Inputkeyconf[i].GPIOx;
		GPIO_Pin = Inputkeyconf[i].GPIO_Pin;
		if(!GPIO_ReadInputDataBit(GPIOx,GPIO_Pin))
			debouncebuf[i] = debouncebuf[i]<<1 | 0x1;
		else
			debouncebuf[i] = debouncebuf[i]<<1 | 0x0;

		if ( (~debouncebuf[i] & 0xF ) == 00)
			isKeypress = true;
		else
			isKeypress = false;

		// Bei gedrueckter Taste Telegramm zusammenstellen
		if(isKeypress) {
			modifier_key_t modifier; // Ctrl, Alt, Shift & Co.
			uint8_t key;             // Taste
			uint_fast8_t prio;       // Prioritaet der Taste

			modifier = Inputkeyconf[i].modifier;
			key      = Inputkeyconf[i].key;
			prio     = Inputkeyconf[i].prio;

			if(prio<hiPrio) {
				// Neue Taste ist von hoeherer Prioritaet als alle anderen
				// (Zahlenwert kleiner) und ueberschreibt sie
				hiPrio = prio; // jetzt der neue Masstab
				ikey = 0;
				clearKeybuffer();
				keybuffer[0] |= modifier;
				keybuffer[ikey+2] = key;
				ikey++;
			}
			else if(prio>hiPrio) {
				// Neue Taste ist von niegrigerer Prioritaet als irgendeine
				// davor (Zahlenwert groesser) und wird gemeinerweise ignoriert
			}
			else {
				// Neue Taste ist bei den Tasten mit momentan hoechster
				// Prioritaet und reiht sich ein:
				keybuffer[0] |= modifier;
				keybuffer[ikey+2] = key;
				ikey++;
			}
		}

		// Fehlerbedingung fuer mehr als 6 Tasten: Overrun
		if(ikey>=6) {
			clearKeybuffer();
			keybuffer[2] = KEYBOARD_ERROR_ROLL_OVER;
			break;
		}
	}

	return keybuffer;
}






