/*
 * inputkeys.h
 *
 * Konfiguration von Eingangs-Tastern (Pins am C und die auszuloesenden Tasten-
 * kombinationen).
 *
 * Werden mehrere Tasten gedrueckt, gewinnt die Taste mit der hoechsten
 * Prioritaet (niedrigster Zahlenwert fuer "prio"). Haben mehrere Tasten
 * dieselbe hoechste Prioritaet, werden sie gleichzeitig gesendet (bis zu 6
 * Tasten)
 *
 * Liegen mehrere Struct-Elemente aus Inputkeyconf[] auf demselben Taster,
 * werden alle Tasten (mit der normalen Priorisierung) ausgeloest.
 *
 * Konfiguration in inputkey.c, in inputkeys.h sind keine Einstellungen
 * vorzunehmen. Bei Bedarf koennen allerdings Tasten ergaenzt werden.
 *
 * Es wird direkt die HID-Usage-ID der Taste hinterlegt. Das erspart die
 * Implementierung einer kompletten Keymap.
 *
 * 18.09.2014 Nicolas
 */


#ifndef INPUTKEYS_H
	#define INPUTKEYS_H

	#include "stm32f10x_gpio.h"


	// Modifier keys (vollstaendig)
	typedef enum {
		none        = 0,
		LEFT_CTRL   = 1<<0,
		LEFT_SHIFT  = 1<<1,
		LEFT_ALT    = 1<<2,
		LEFT_GUI    = 1<<3,
		RIGHT_CTRL  = 1<<4,
		RIGHT_SHIFT = 1<<5,
		RIGHT_ALT   = 1<<6,
		RIGHT_GUI   = 1<<7
	} modifier_key_t;


	// Keypad Page (Auszug aus HID-Usage-Tables-Dokument 1.12v2, S. 53ff.),
	// (unvollstaendig, kann beliebig ergaenzt werden)
	typedef enum {
		KEYBOARD_ERROR_ROLL_OVER = 0x01,

		KEY_A      = 0x04,
		KEY_B      = 0x05,
		KEY_C      = 0x06,
		KEY_D      = 0x07,
		KEY_E      = 0x08,
		KEY_F      = 0x09,
		KEY_G,
		KEY_H,
		KEY_I,
		KEY_J,

		KEY_R      = 0x15,
		KEY_S      = 0x16,

		KEY_W      = 0x1A,
		KEY_X      = 0x1B,
		KEY_Y      = 0x1C,
		KEY_Z      = 0x1D,

		KEY_RETURN = 0x28, // Enter
		KEY_ESCAPE = 0x29,
		KEY_DELETE = 0x2A, // Backspace
		KEY_TAB    = 0x2B,
		KEY_SPACE  = 0x2C,

		KEY_F1     = 0x3A,
		KEY_F2     = 0x3B,
		KEY_F3     = 0x3C,
		KEY_F4     = 0x3D,
		KEY_F5     = 0x3E,
		KEY_F6     = 0x3F,
		KEY_F7     = 0x40,
		KEY_F8     = 0x41,
		KEY_F9     = 0x42,
		KEY_F10    = 0x43,
		KEY_F11    = 0x44,
		KEY_F12    = 0x45,

		KEY_PAGE_UP = 0x4B,
		KEY_DELETE_FORWARD = 0x4C, // Del, Entf
		KEY_END    = 0x4D,
		KEY_PAGE_DOWN = 0x4E,
		KEY_RIGHT_ARROW = 0x4F,
		KEY_LEFT_ARROW = 0x50,
		KEY_DOWN_ARROW = 0x51,
		KEY_UP_ARROW = 0x52,

	} keymap_t;


	// Taste am C und ihre Zuordnung zu auszuloesenden Tastenkombinationen
	typedef struct {
		GPIO_TypeDef* GPIOx;     // Input-Port
		uint16_t GPIO_Pin;       // Input-Pin
		uint8_t prio;            // 0: hoechste , 255: niedrigste Prioritaet
		modifier_key_t modifier; // z.B. Alt, Strg, ...
		keymap_t key;            // Taste (als HID-Scancode)
		uint8_t active_modes;    // Betriebsmodi, in denen Taste aktiv 0xFF = all
	} inputkeyconf_t;


	// Konfigurationsdaten
	extern const inputkeyconf_t Inputkeyconf[];

	void inputkeys_init(void);
	uint8_t * inputkeys_scan(void);


#endif
