/*
 * usb_prop.c
 *
 * Benutzerfunktionen fuer USB
 *
 * basiert auf der gleichnamigen Datei des Joystick-Demos aus der
 * USB-FS-Library 4.0.0 sowie der Datei hwconfig.c aus demselben Demo.
 *
 * 17.09.2014 Nicolas
 */




/* Includes ------------------------------------------------------------------*/
#include "usb_lib.h"
#include "usb_conf.h"
#include "usb_prop.h"
#include "usb_pwr.h"
#include "hw_config.h"
#include "keyboard_usb_desc.h"
#include "debughelp.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
uint32_t ProtocolValue;
uint8_t Report_Buf[8];


/* -------------------------------------------------------------------------- */
/*  Structures initializations */
/* -------------------------------------------------------------------------- */

DEVICE Device_Table =
  {
    EP_NUM,
    1
  };


/* Wird von USB_Init in "pProperty" hinterlegt */
DEVICE_PROP Device_Property =
  {
    usbdev_init,
    usbdev_reset,
    usbdev_status_In,
    usbdev_status_Out,
    usbdev_data_Setup,
    usbdev_noData_Setup,
    usbdev_Get_Interface_Setting,
    usbdev_getDeviceDescriptor,
    usbdev_getConfigDescriptor,
    usbdev_getStringDescriptor,
    0,
    0x40 /*MAX PACKET SIZE*/
  };


/* Wird von USB_Init in "pUser_Standard_Requests" hinterlegt */
USER_STANDARD_REQUESTS User_Standard_Requests =
  {
    usbdev_getConfiguration,
    usbdev_setConfiguration,
    usbdev_getInterface,
    usbdev_setInterface,
    usbdev_getStatus,
    usbdev_clearFeature,
    usbdev_setEndPointFeature,
    usbdev_setDeviceFeature,
    usbdev_setDeviceAddress
  };


/* Mundgerechte Darstellungen fuer Standard_GetDescriptorData in usb_core.c */
ONE_DESCRIPTOR Device_Descriptor =
  {
    (uint8_t*)usbdev_DeviceDescriptor,
    USBDEV_SIZ_DEVICE_DESC
  };

ONE_DESCRIPTOR Config_Descriptor =
  {
    (uint8_t*)usbdev_ConfigDescriptor,
    USBDEV_SIZ_CONFIG_DESC
  };

ONE_DESCRIPTOR Report_Descriptor =
  {
    (uint8_t *)usbdev_ReportDescriptor,
    USBDEV_SIZ_REPORT_DESC
  };

ONE_DESCRIPTOR Hid_Descriptor =
  {
    (uint8_t*)usbdev_ConfigDescriptor + USBDEV_OFF_HID_DESC,
    USBDEV_SIZ_HID_DESC
  };

ONE_DESCRIPTOR String_Descriptor[4] =
  {
    {(uint8_t*)usbdev_StringLangID, USBDEV_SIZ_STRING_LANGID},
    {(uint8_t*)usbdev_StringVendor, USBDEV_SIZ_STRING_VENDOR},
    {(uint8_t*)usbdev_StringProduct, USBDEV_SIZ_STRING_PRODUCT},
    {(uint8_t*)usbdev_StringSerial, USBDEV_SIZ_STRING_SERIAL}
  };


/* Extern variables ----------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
static void Get_SerialNum(void);
static void IntToUnicode (uint32_t value , uint8_t *pbuf , uint8_t len);
/* Extern function prototypes ------------------------------------------------*/
/* Private functions ---------------------------------------------------------*/



/* Init-Routine fuer USB-HID-Device */
void usbdev_init(void)
{
	// Aufruf loggen (bei Debug-Build)
	logit(f_usbdev_init,0);

	/* Update the serial number string descriptor with the data from the unique
  	  ID*/
	Get_SerialNum();

	pInformation->Current_Configuration = 0;

	/* Connect the device */
	PowerOn();

	/* Perform basic device initialization operations */
	USB_SIL_Init();

	bDeviceState = UNCONNECTED;
}


/* Reset-Routine fuer USB-HID-Device
 * beim Anmelden am PC wird dieser Reset sehr oft hintereinander ausgefuehrt. */
void usbdev_reset(void)
{
	// Aufruf loggen (bei Debug-Build)
	logit(f_usbdev_reset,0);

	/* Set Joystick_DEVICE as not configured */
	pInformation->Current_Configuration = 0;
	pInformation->Current_Interface = 0;/*the default Interface*/

	/* Current Feature initialization */
	pInformation->Current_Feature = usbdev_ConfigDescriptor[7]; //bmAttributes
	SetBTABLE(BTABLE_ADDRESS);

	/* Initialize Endpoint 0 */
	/* Endpoint 0 dient nur der Konfiguration */
	SetEPType(ENDP0, EP_CONTROL);
	SetEPTxStatus(ENDP0, EP_TX_STALL);
	SetEPRxAddr(ENDP0, ENDP0_RXADDR);
	SetEPTxAddr(ENDP0, ENDP0_TXADDR);
	Clear_Status_Out(ENDP0);
	SetEPRxCount(ENDP0, Device_Property.MaxPacketSize);
	SetEPRxValid(ENDP0);

	/* Initialize Endpoint 1 (hier: Tastatur) */
	SetEPType(ENDP1, EP_INTERRUPT);
	SetEPTxAddr(ENDP1, ENDP1_TXADDR);
	SetEPTxCount(ENDP1, 8);

	SetEPRxAddr(ENDP1, ENDP1_RXADDR);
	SetEPRxCount(ENDP1, 8);

	SetEPRxStatus(ENDP1, EP_RX_DIS);
    SetEPTxStatus(ENDP1, EP_TX_NAK);

    /* Set this device to response on default address */
    SetDeviceAddress(0);
    bDeviceState = ATTACHED;
}


/* Update the device state to configured.
 * Bis auf Log-Funktion unveraendert gegenueber dem Demo
 * (Joystick_SetConfiguration) */
void usbdev_setConfiguration(void)
{
	// Aufruf loggen (bei Debug-Build)
	logit(f_usbdev_setConfiguration,0);

	DEVICE_INFO *pInfo = &Device_Info;

	if (pInfo->Current_Configuration != 0)
	{
		/* Device configured */
		bDeviceState = CONFIGURED;
	}
}


/* Update the device state to addressed.
 * Bis auf Log-Funktion unveraendert gegenueber dem Demo
 * (Joystick_SetDeviceAddress) */
void usbdev_setDeviceAddress (void)
{
	// Aufruf loggen (bei Debug-Build)
	logit(f_usbdev_setDeviceAddress,0);

	bDeviceState = ADDRESSED;
}


/* Macht nichts (ausser ihren Aufruf loggen)
 * Bis auf Log-Funktion unveraendert gegenueber dem Demo
 * (Joystick_Status_In) */
void usbdev_status_In(void)
{
	// Aufruf loggen (bei Debug-Build)
	logit(f_usbdev_status_In,0);
}


/* Macht nichts (ausser ihren Aufruf loggen)
 * Insbesondere ist diese Funktion nicht fuer den Empfang der SET_REPORT-Pakete
 * zustaendig.
 * Bis auf Log-Funktion unveraendert gegenueber dem Demo
 * (Joystick_Status_Out) */
void usbdev_status_Out (void)
{
	// Aufruf loggen (bei Debug-Build)
	logit(f_usbdev_status_Out,0);
}


/* Handle the data class specific requests.
 * RequestNo:
 * return   : USB_UNSUPPORT oder USB_SUCCESS
 * Funktion stark gegenueber dem Demo (Joystick_Data_Setup) geaendert, um auch
 * andere Requests zu unterstuetzen.  */
RESULT usbdev_data_Setup(uint8_t RequestNo)
{
	// Aufruf inkl. Parameter loggen (bei Debug-Build)
	logit(f_usbdev_data_Setup,RequestNo);

	uint8_t *(*CopyRoutine)(uint16_t);
	CopyRoutine = NULL;

	if ((RequestNo == GET_DESCRIPTOR)
		&& (Type_Recipient == (STANDARD_REQUEST | INTERFACE_RECIPIENT))
        && (pInformation->USBwIndex0 == 0))
	{
		if (pInformation->USBwValue1 == REPORT_DESCRIPTOR)
		{
			CopyRoutine = usbdev_getReportDescriptor;
		}
		else if (pInformation->USBwValue1 == HID_DESCRIPTOR_TYPE)
		{
			CopyRoutine = usbdev_getHIDDescriptor;
		}
    } /* End of GET_DESCRIPTOR */

    // Andere Requests
    else if ((Type_Recipient == (CLASS_REQUEST | INTERFACE_RECIPIENT)) )
        switch(RequestNo) {
        case GET_REPORT:
        	error(); // passiert das?
        	break;
        case GET_IDLE:
        	logit(fGetIdle,RequestNo);
        	break;
        case GET_PROTOCOL:
        	CopyRoutine = usbdev_getProtocolValue;
        	break;
        case SET_REPORT:
        	logit(fSetReport,RequestNo);
        	// Eine Aenderung des Num-Lock-Status ruft Data_Setup mit einem
        	// SET_REPORT auf
            CopyRoutine = usbdev_setReportFeature;
        	break;
        case SET_IDLE:
        	error(); // passiert das?
        	break;
        case SET_PROTOCOL:
        	error(); // passiert das?
        }

	if (CopyRoutine == NULL)
	{
		return USB_UNSUPPORT;
	}

	pInformation->Ctrl_Info.CopyData = CopyRoutine;
	pInformation->Ctrl_Info.Usb_wOffset = 0;
	(*CopyRoutine)(0);
	return USB_SUCCESS;
}


/* handle the no data class specific requests
 * RequestNo : ??
 * return: USB_UNSUPPORT oder USB_SUCCESS.
 * Nur minimale Aenderungen gegenueber dem Demo (Joystick_NoData_Setup) */
RESULT usbdev_noData_Setup(uint8_t RequestNo)
{
	// Aufruf inkl. Parameter loggen (bei Debug-Build)
	logit(f_usbdev_noData_Setup,RequestNo);

	if ((Type_Recipient == (CLASS_REQUEST | INTERFACE_RECIPIENT)) )
	{
		switch(RequestNo)
		{
		case SET_PROTOCOL:
			return usbdev_setProtocol();
		case SET_IDLE:
			// noch nicht implementiert
			break;

		default:
			error();
		}
	}
	else
	{
		return USB_UNSUPPORT;
	}
}


/*******************************************************************************
* Function Name  : Joystick_GetDeviceDescriptor.
* Description    : Gets the device descriptor.
* Input          : Length
* Output         : None.
* Return         : The address of the device descriptor.
*******************************************************************************/
// Nur Logging hinzugefuegt
uint8_t *usbdev_getDeviceDescriptor(uint16_t Length)
{
	logit(f_usbdev_getDeviceDescriptor,Length);
	return Standard_GetDescriptorData(Length, &Device_Descriptor);
}


/*******************************************************************************
* Function Name  : Joystick_GetConfigDescriptor.
* Description    : Gets the configuration descriptor.
* Input          : Length
* Output         : None.
* Return         : The address of the configuration descriptor.
*******************************************************************************/
// Nur Logging hinzugefuegt
uint8_t *usbdev_getConfigDescriptor(uint16_t Length)
{
	logit(f_usbdev_getConfigDescriptor,Length);
	return Standard_GetDescriptorData(Length, &Config_Descriptor);
}


/*******************************************************************************
* Function Name  : Joystick_GetStringDescriptor
* Description    : Gets the string descriptors according to the needed index
* Input          : Length
* Output         : None.
* Return         : The address of the string descriptors.
*******************************************************************************/
// Nur Logging hinzugefuegt
uint8_t *usbdev_getStringDescriptor(uint16_t Length)
{
	logit(f_usbdev_GetStringDescriptor,Length);

	uint8_t wValue0 = pInformation->USBwValue0;
	if (wValue0 > 4)
	{
		return NULL;
	}
	else
	{
		return Standard_GetDescriptorData(Length, &String_Descriptor[wValue0]);
	}
}


/*******************************************************************************
* Function Name  : Joystick_GetReportDescriptor.
* Description    : Gets the HID report descriptor.
* Input          : Length
* Output         : None.
* Return         : The address of the configuration descriptor.
*******************************************************************************/
// Nur Logging hinzugefuegt
uint8_t *usbdev_getReportDescriptor(uint16_t Length)
{
	logit(f_usbdev_getReportDescriptor,Length);
	return Standard_GetDescriptorData(Length, &Report_Descriptor);
}


/*******************************************************************************
* Function Name  : Joystick_GetHIDDescriptor.
* Description    : Gets the HID descriptor.
* Input          : Length
* Output         : None.
* Return         : The address of the configuration descriptor.
*******************************************************************************/
// Nur Logging hinzugefuegt
uint8_t *usbdev_getHIDDescriptor(uint16_t Length)
{
	logit(f_usbdev_getHIDDescriptor,Length);
	return Standard_GetDescriptorData(Length, &Hid_Descriptor);
}


/*******************************************************************************
* Function Name  : Joystick_Get_Interface_Setting.
* Description    : tests the interface and the alternate setting according to
*                  the supported one.
* Input          : - Interface : interface number.
*                  - AlternateSetting : Alternate Setting number.
* Output         : None.
* Return         : USB_SUCCESS or USB_UNSUPPORT.
*******************************************************************************/
// Nur Logging hinzugefuegt
RESULT usbdev_Get_Interface_Setting(uint8_t Interface, uint8_t AlternateSetting)
{
	logit(f_usbdev_get_Interface_Setting,Interface<<8 | AlternateSetting);

	if (AlternateSetting > 0)
	{
		return USB_UNSUPPORT;
	}
	else if (Interface > 0)
	{
		return USB_UNSUPPORT;
	}
	return USB_SUCCESS;
}


/*******************************************************************************
* Function Name  : Joystick_SetProtocol
* Description    : Joystick Set Protocol request routine.
* Input          : None.
* Output         : None.
* Return         : USB SUCCESS.
*******************************************************************************/
// Nur Logging hinzugefuegt
RESULT usbdev_setProtocol(void)
{
	logit(f_usbdev_setProtocol,0);
  uint8_t wValue0 = pInformation->USBwValue0;
  ProtocolValue = wValue0;
  return USB_SUCCESS;
}


/*******************************************************************************
* Function Name  : Joystick_GetProtocolValue
* Description    : get the protocol value
* Input          : Length.
* Output         : None.
* Return         : address of the protocol value.
*******************************************************************************/
// Nur Logging hinzugefuegt
uint8_t *usbdev_getProtocolValue(uint16_t Length)
{
	logit(f_usbdev_getProtocolValue,Length);
	if (Length == 0)
	{
		pInformation->Ctrl_Info.Usb_wLength = 1;
		return NULL;
	}
	else
	{
		return (uint8_t *)(&ProtocolValue);
	}
}


/*******************************************************************************
* Function Name  : CustomHID_SetReport_Feature
* Description    : Set Feature request handling
* Input          : Length.
* Output         : None.
* Return         : Buffer
*******************************************************************************/
// Aus: CustomHID-Demo, usb_prop.c
// Nur Logging hinzugefuegt
uint8_t *usbdev_setReportFeature(uint16_t Length)
{
	if (Length == 0)
	{
		pInformation->Ctrl_Info.Usb_wLength = 2;
		return NULL;
	}
	else
	{
		return Report_Buf;
	}
}


/*******************************************************************************
* Function Name  : Get_SerialNum.
* Description    : Create the serial number string descriptor.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
// Aus: hw_config.c; unveraendert
static void Get_SerialNum(void)
{
	uint32_t Device_Serial0, Device_Serial1, Device_Serial2;

	Device_Serial0 = *(uint32_t*)ID1;
	Device_Serial1 = *(uint32_t*)ID2;
	Device_Serial2 = *(uint32_t*)ID3;

	Device_Serial0 += Device_Serial2;

	if (Device_Serial0 != 0)
	{
		IntToUnicode (Device_Serial0, &usbdev_StringSerial[2] , 8);
		IntToUnicode (Device_Serial1, &usbdev_StringSerial[18], 4);
	}
}

/*******************************************************************************
* Function Name  : HexToChar.
* Description    : Convert Hex 32Bits value into char.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
// Aus: hw_config.c; unveraendert
static void IntToUnicode (uint32_t value , uint8_t *pbuf , uint8_t len)
{
	uint8_t idx = 0;

	for( idx = 0 ; idx < len ; idx ++)
	{
		if( ((value >> 28)) < 0xA )
		{
			pbuf[ 2* idx] = (value >> 28) + '0';
		}
		else
		{
			pbuf[2* idx] = (value >> 28) + 'A' - 10;
		}

		value = value << 4;

		pbuf[ 2* idx + 1] = 0;
	}
}


/************************ (C) COPYRIGHT ??? *****END OF FILE****/
